'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2006
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System.Collections.Generic

Namespace DotNetNuke.Modules.Reports

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' The Packages class manages Reports Module Extension Packages (EPs)
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Partial Class Packages
        Inherits Entities.Modules.PortalModuleBase

        Private ReadOnly Property InstallerResourceFile()
            Get
                Return String.Format("{0}/{1}/Installer.ascx", _
                    Me.TemplateSourceDirectory, Localization.LocalResourceDirectory)
            End Get
        End Property

        Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
            If Not IsPostBack Then
                ConfigureGrid()
            End If
            PreviousVersionWarningBlock.Visible = False
            DataBind()
        End Sub

        Public Overrides Sub DataBind()
            Me.PackagesGrid.DataSource = Installer.PackageInstaller.GetExtensions(Server.MapPath(Me.TemplateSourceDirectory))

            MyBase.DataBind()
        End Sub

        Private Sub ConfigureGrid()
            Dim delButtonColumn As New CommandField()
            delButtonColumn.ShowEditButton = False
            delButtonColumn.ShowCancelButton = False
            delButtonColumn.ShowInsertButton = False
            delButtonColumn.ShowSelectButton = False
            delButtonColumn.ShowDeleteButton = True
            delButtonColumn.DeleteImageUrl = "~/images/delete.gif"
            delButtonColumn.ButtonType = ButtonType.Image
            Me.PackagesGrid.Columns.Add(delButtonColumn)

            Me.PackagesGrid.Columns.Add(BuildBoundColumn("Name", Unit.Pixel(125)))
            Me.PackagesGrid.Columns.Add(BuildBoundColumn("PackageVersion", Unit.Pixel(50)))
            Me.PackagesGrid.Columns.Add(BuildBoundColumn("Type", Unit.Pixel(75)))
            Me.PackagesGrid.Columns.Add(BuildBoundColumn("Vendor", Unit.Pixel(125)))
            Me.PackagesGrid.Columns.Add(BuildBoundColumn("Description", Unit.Empty))
        End Sub

        Private Function BuildBoundColumn(ByVal DataField As String, ByVal Width As Unit) As BoundField
            Return BuildBoundColumn(String.Concat(DataField, ".HeaderText"), DataField, Width)
        End Function

        Private Function BuildBoundColumn(ByVal HeaderKey As String, ByVal DataField As String, ByVal Width As Unit) As BoundField
            Dim bf As New BoundField
            bf.HeaderText = Localization.GetString(HeaderKey, Me.LocalResourceFile)
            bf.HeaderStyle.CssClass = "SubHead"
            bf.HeaderStyle.Width = Width
            bf.ItemStyle.CssClass = "Normal"
            bf.ItemStyle.Width = Width
            bf.DataField = DataField
            Return bf
        End Function

        Protected Sub InstallButton_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles InstallButton.Click
            ' Install the Package
            Dim success As Boolean = False
            Try
                Installer.PackageInstaller.InstallPackage(Me.PackageFileUpload.PostedFile.InputStream, Me.TemplateSourceDirectory)
                success = True
            Catch ex As Installer.InstallerException
                ' Show a module message with the error message
                Dim message As String = Localization.GetString(ex.ErrorMessageKey, Me.InstallerResourceFile)
                If Not String.IsNullOrEmpty(ex.Argument) Then
                    message = String.Format(message, ex.Argument)
                End If
                If ex.InnerException IsNot Nothing Then
                    message = String.Format("{0}. Error Details: {1}", message, ex.InnerException.Message)
                End If
                DotNetNuke.UI.Skins.Skin.AddModuleMessage(Me, message, Skins.Controls.ModuleMessage.ModuleMessageType.RedError)
                success = False
                'Throw ' For Testing only
            End Try
            If success Then
                DotNetNuke.UI.Skins.Skin.AddModuleMessage(Me, Localization.GetString("InstallationSucceeded.Message", Me.InstallerResourceFile), Skins.Controls.ModuleMessage.ModuleMessageType.GreenSuccess)
                DataBind()
            End If
        End Sub

        Protected Sub PackagesGrid_RowDeleting(ByVal sender As Object, ByVal e As System.Web.UI.WebControls.GridViewDeleteEventArgs) Handles PackagesGrid.RowDeleting
            Dim success As Boolean = False
            Dim row As GridViewRow = PackagesGrid.Rows(e.RowIndex)
            Try
                Dim manifests As List(Of Installer.PackageManifest) = DirectCast( _
                    Me.PackagesGrid.DataSource, List(Of Installer.PackageManifest))
                Dim manifest As Installer.PackageManifest = manifests( _
                    row.DataItemIndex)
                Installer.PackageInstaller.UninstallPackage(Me.TemplateSourceDirectory, manifest)
                success = True
            Catch ex As Installer.InstallerException
                ' Show a module message with the error message
                Dim message As String = Localization.GetString(ex.ErrorMessageKey, Me.InstallerResourceFile)
                If Not String.IsNullOrEmpty(ex.Argument) Then
                    message = String.Format(message, ex.Argument)
                End If
                If ex.InnerException IsNot Nothing Then
                    message = String.Format("{0}.<br/>Error Details: {1}", message, ex.InnerException.Message)
                End If
                DotNetNuke.UI.Skins.Skin.AddModuleMessage(Me, message, Skins.Controls.ModuleMessage.ModuleMessageType.YellowWarning)
                success = False
                'Throw ' For Testing only
            End Try
            If success Then
                DotNetNuke.UI.Skins.Skin.AddModuleMessage(Me, Localization.GetString("UninstallationSucceeded.Message", Me.InstallerResourceFile), Skins.Controls.ModuleMessage.ModuleMessageType.GreenSuccess)
            End If
            DataBind()
        End Sub

        Protected Sub PackagesGrid_RowDataBound(ByVal sender As Object, ByVal e As System.Web.UI.WebControls.GridViewRowEventArgs) Handles PackagesGrid.RowDataBound
            If (e.Row.Cells.Count < 1) OrElse (e.Row.Cells(0).Controls.Count < 1) Then Return

            Dim img As ImageButton = TryCast(e.Row.Cells(0).Controls(0), ImageButton)

            If img Is Nothing Then Return

            img.Attributes.Add("onclick", DeleteButtonConfirm())
        End Sub

        Protected Function LocalString(ByVal Key As String) As String
            Return Localization.GetString(Key, Me.LocalResourceFile)
        End Function

        Protected Function DeleteButtonConfirm() As String
            Return String.Format("javascript:if(!confirm('{0}')) return false;", _
                Localization.GetString("RemoveExtension.Confirm", LocalResourceFile))
        End Function

        Protected Function IsLegacy(ByVal version As Version) As Boolean
            If ((version.Major <> 4) OrElse (version.Minor <> 5)) Then
                PreviousVersionWarningBlock.Visible = True
                Return True
            Else
                Return False
            End If
        End Function

    End Class

End Namespace
