'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2006
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System
Imports System.Configuration
Imports System.Data
Imports System.Data.Common
Imports System.IO
Imports System.Xml
Imports System.Web
Imports System.Collections.Generic
Imports DotNetNuke
Imports DotNetNuke.Entities.Modules
Imports DotNetNuke.Services.Search
Imports DotNetNuke.Common.Utilities.XmlUtils
Imports DotNetNuke.Modules.Reports.Installer

Imports ReportsDataProvider = DotNetNuke.Modules.Reports.Data.DataProvider
Imports ICSharpCode.SharpZipLib.Zip

Namespace DotNetNuke.Modules.Reports

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' The Controller class for Reports
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[anurse]	06/16/2006	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Class ReportsController
        Implements Entities.Modules.ISearchable
        Implements Entities.Modules.IPortable

#Region " Public Constants "

        ' Prefixes
        Public Const PREFIX_Reports As String = "dnn_Reports_"
        Public Const PREFIX_Visualizer As String = "dnn_ReportsVis_"

        ' Reports Module Settings
        Public Const SETTING_ReportTitle As String = "dnn_Reports_Title"
        Public Const SETTING_ReportDescription As String = "dnn_Reports_Description"
        Public Const SETTING_Query As String = "dnn_Reports_Query"
        Public Const SETTING_ReportCreatedBy As String = "dnn_Reports_CreatedBy"
        Public Const SETTING_ReportCreatedOn As String = "dnn_Reports_CreatedOn"
        Public Const SETTING_CacheDuration As String = "dnn_Reports_CacheDuration"
        Public Const SETTING_Visualizer As String = "dnn_Reports_Visualizer"


        ' Visualizer Settings

        ' Common
        Public Const SETTING_Height As String = "Height"
        Public Const SETTING_Width As String = "Width"

        ' Chart Visualizer
        Public Const SETTING_Chart_Type As String = "Type"
        Public Const SETTING_Chart_BarNameColumn As String = "BarNameColumn"
        Public Const SETTING_Chart_BarValueColumn As String = "BarValueColumn"
        Public Const SETTING_Chart_ColorMode As String = "ColorMode"
        Public Const SETTING_Chart_BarColorColumn As String = "BarColorColumn"
        Public Const SETTING_Chart_BarColor As String = "BarColor"
        Public Const SETTING_Chart_XAxisTitle As String = "XAxisTitle"
        Public Const SETTING_Chart_YAxisTitle As String = "YAxisTitle"

        ' Grid Visualizer
        Public Const SETTING_Grid_EnablePaging As String = "EnablePaging"
        Public Const SETTING_Grid_EnableSorting As String = "EnableSorting"
        Public Const SETTING_Grid_PageSize As String = "PageSize"
        Public Const SETTING_Grid_ShowHeader As String = "ShowHeader"

        ' Reporting Services Visualizer
        Public Const SETTING_RS_Mode As String = "Mode"
        Public Const SETTING_RS_LocalReportFile As String = "LocalFile"
        Public Const SETTING_RS_ServerReportUrl As String = "ServerUrl"
        Public Const SETTING_RS_ServerReportPath As String = "ServerPath"
        Public Const SETTING_RS_DataSourceName As String = "DataSourceName"

        ' HTML Visualizer
        Public Const SETTING_Html_TemplateFile As String = "TemplateFile"
        Public Const SETTING_Html_Repeat As String = "Repeat"
        Public Const SETTING_Html_Separator As String = "Separator"

        ' XSLT Visualizer
        Public Const SETTING_Xslt_TransformFile As String = "TransformFile"

        Public Const CACHEKEY_Reports As String = "dnn_ReportCache"

#End Region

#Region " Public Methods "

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets the report associated with a reports module without retrieving
        ''' tab module specific settings (visualizers)
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <param name="ModuleId">The Id of the module to retrive the report for</param>
        ''' <history>
        ''' 	[anurse]	06/16/2006	Created
        ''' 	[anurse]	01/15/2007	Updated to use GetReport(Integer, Integer) override
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetReport(ByVal ModuleId As Integer) As ReportInfo
            Return GetReport(ModuleId, Null.NullInteger)
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets the report associated with a reports module (including tab module
        ''' specific settings)
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <param name="ModuleId">The Id of the module</param>
        ''' <param name="TabModuleId">
        ''' The Id of the instance ("TabModule") of the module that the report is running on
        ''' </param>
        ''' <history>
        ''' 	[anurse]	01/15/2007	Created from GetReport(Integer)
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetReport(ByVal ModuleId As Integer, ByVal TabModuleId As Integer) As ReportInfo

            ' Check for a null Module Id
            If ModuleId = Null.NullInteger Then Return Nothing

            ' Extract the Title, Description and Query from the settings
            Dim objModuleController As New ModuleController
            Dim objSettings As Hashtable = _
                objModuleController.GetModuleSettings(ModuleId)
            ' Check that the settings hashtable was retrieved
            If objSettings Is Nothing Then Return Nothing

            ' Setup the Report
            Dim objReport As New ReportInfo()

            ' Build the Report from the Module Settings
            objReport.Query = SettingsUtil.GetHashtableSetting(Of String)(objSettings, ReportsController.SETTING_Query, Null.NullString)
            objReport.Title = SettingsUtil.GetHashtableSetting(Of String)(objSettings, ReportsController.SETTING_ReportTitle, Null.NullString)
            objReport.Description = SettingsUtil.GetHashtableSetting(Of String)(objSettings, ReportsController.SETTING_ReportDescription, Null.NullString)
            objReport.CreatedOn = SettingsUtil.GetHashtableSetting(Of Date)(objSettings, ReportsController.SETTING_ReportCreatedOn, Null.NullDate)
            objReport.CreatedBy = SettingsUtil.GetHashtableSetting(Of Integer)(objSettings, ReportsController.SETTING_ReportCreatedBy, Null.NullInteger)
            objReport.ModuleID = ModuleId

            ' Load the tab module settings (visualizer settings) if we have a tab module Id
            If TabModuleId <> Null.NullInteger Then

                Dim objTabModuleSettings As Hashtable = _
                    objModuleController.GetTabModuleSettings(TabModuleId)

                ' Read the visualizer name and cache duration
                objReport.Visualizer = SettingsUtil.GetHashtableSetting(Of String)(objTabModuleSettings, ReportsController.SETTING_Visualizer, Null.NullString)
                objReport.CacheDuration = SettingsUtil.GetHashtableSetting(Of Integer)(objTabModuleSettings, ReportsController.SETTING_CacheDuration, Null.NullInteger)

                ' Set the visualizer to a default if it is not specified
                If String.IsNullOrEmpty(objReport.Visualizer) Then
                    objReport.Visualizer = "Grid"
                End If

                ' Build the visualizer prefix: dnn_ReportsVis_[VisualizerName]_
                Dim strVisPrefix = String.Format("{0}{1}_", PREFIX_Visualizer, objReport.Visualizer)

                ' Read the tab module settings
                For Each key As Object In objTabModuleSettings.Keys

                    Dim sKey As String = DirectCast(key, String)

                    ' If the setting starts with the current visualizer's prefix
                    If sKey.StartsWith(strVisPrefix) Then

                        ' Strip out the prefix and add it to the visualizer settings
                        objReport.VisualizerSettings.Add( _
                            sKey.Substring(strVisPrefix.Length), _
                            SettingsUtil.GetHashtableSetting(Of String)(objTabModuleSettings, sKey, Null.NullString))
                    End If
                Next
            End If

            Return objReport

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Saves the report associated with a reports module
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <param name="ModuleId">The ModuleId to set the report for</param>
        ''' <param name="TabModuleId">
        ''' The Id of the instance ("TabModule") of the module to store the visualizer
        ''' settings with
        ''' </param>
        ''' <param name="objReport">The ReportInfo object to save</param>
        ''' <history>
        ''' 	[anurse]	06/16/2006	Created
        ''' 	[anurse]	01/15/2007	Refactored, creating SaveReportDefinition and
        '''                             SaveReportViewSettings.
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Sub UpdateReport(ByVal ModuleId As Integer, ByVal TabModuleId As Integer, _
            ByVal objReport As ReportInfo)

            Dim objModuleController As New ModuleController
            UpdateReportDefinition(ModuleId, objReport)
            UpdateReportView(TabModuleId, objReport)

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Saves the report definition (module settings) associated with a reports module
        ''' </summary>
        ''' <param name="ModuleId">The ModuleId to set the report for</param>
        ''' <param name="objReport">The ReportInfo object to save</param>
        ''' <history>
        ''' 	[anurse]	01/15/2007	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Sub UpdateReportDefinition(ByVal ModuleId As Integer, ByVal objReport As ReportInfo)
            Dim ctrl As New ModuleController()
            UpdateReportDefinition(ctrl, ModuleId, objReport)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Saves the report view (tab module settings) associated with a reports module
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <param name="TabModuleId">
        ''' The Id of the instance ("TabModule") of the module to store the view
        ''' settings for.
        ''' </param>
        ''' <param name="objReport">The ReportInfo object to save</param>
        ''' <history>
        ''' 	[anurse]	01/15/2007	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Sub UpdateReportView(ByVal TabModuleId As Integer, ByVal objReport As ReportInfo)
            Dim ctrl As New ModuleController()
            UpdateReportView(ctrl, TabModuleId, objReport)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Executes a report and returns the results
        ''' </summary>
        ''' <param name="objReport">The ReportInfo object</param>
        ''' <param name="cacheKey">The cache key for the module</param>
        ''' <exception cref="System.ArgumentNullException">
        ''' The value of <paramref name="objReport"/> was null (Nothing in Visual Basic)
        ''' </exception>
        ''' <history>
        ''' 	[anurse]	06/16/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function ExecuteReport(ByVal objReport As ReportInfo, ByVal cacheKey As String) As DataTable
            Return ExecuteReport(objReport, cacheKey, False, Nothing)
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Executes a report and returns the results
        ''' </summary>
        ''' <param name="objReport">The ReportInfo object</param>
        ''' <param name="cacheKey">The cache key for the module</param>
        ''' <param name="bypassCache">A boolean indicating if the controller should bypass the cache completely</param>
        ''' <exception cref="System.ArgumentNullException">
        ''' The value of <paramref name="objReport"/> was null (Nothing in Visual Basic)
        ''' </exception>
        ''' <history>
        ''' 	[anurse]	06/16/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function ExecuteReport(ByVal objReport As ReportInfo, ByVal cacheKey As String, ByVal bypassCache As Boolean) As DataTable
            Return ExecuteReport(objReport, cacheKey, bypassCache, Nothing)
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Executes a report and returns the results
        ''' </summary>
        ''' <param name="objReport">The ReportInfo object</param>
        ''' <param name="cacheKey">The cache key for the module</param>
        ''' <param name="bypassCache">A boolean indicating if the controller should bypass the cache completely</param>
        ''' <exception cref="System.ArgumentNullException">
        ''' The value of <paramref name="objReport"/> was null (Nothing in Visual Basic)
        ''' </exception>
        ''' <history>
        ''' 	[anurse]	06/16/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function ExecuteReport(ByVal objReport As ReportInfo, ByVal cacheKey As String, ByVal bypassCache As Boolean, ByVal ParamArray parameters() As DbParameter) As DataTable
            If objReport Is Nothing Then Return Nothing

            If String.IsNullOrEmpty(objReport.Query) Then
                Return Nothing ' If there's no query, there's no report
            Else
                Dim objCache As Object = DataCache.GetCache(cacheKey)
                Dim dataTable As DataTable = Nothing
                If Not bypassCache AndAlso objCache IsNot Nothing AndAlso TypeOf objCache Is DataTable Then
                    dataTable = DirectCast(objCache, DataTable)
                Else
                    Dim dr As IDataReader = ReportsDataProvider.Instance().ExecuteSQL(objReport.Query, parameters)
                    dataTable = New DataTable("QueryResults")
                    If dr Is Nothing Then Return dataTable
                    dataTable.Load(dr)
                    DataCache.SetCache(cacheKey, dataTable, DateTime.Now.AddMinutes(objReport.CacheDuration))
                End If
                Return dataTable
            End If

        End Function

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Removes "Bad" SQL Commands from the specified string
        ''' </summary>
        ''' <param name="strSQL">This is the string to be filtered</param>
        ''' <returns>A filtered version of <paramref name="strSQL" /> with commands such as INSERT or DELETE removed</returns>
        ''' <history>
        '''     [anurse]        6/20/2006   Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Shared Function FormatRemoveSQL(ByVal strSQL As String) As String

            Dim strCleanSQL As String = strSQL

            If strSQL <> Nothing Then

                ' each string in this array is one that must be removed from the SQL
                Dim BadSQL As String() = New String() {";", "--", "create ", "drop ", "insert ", "delete ", "update ", "sp_", "xp_"}

                ' strip any dangerous SQL commands
                Dim intCommand As Integer
                For intCommand = 0 To BadSQL.Length - 1
                    ' remove the current item in the "Bad SQL" list from the string by replacing it with a space
                    strCleanSQL = Regex.Replace(strCleanSQL, Convert.ToString(BadSQL.GetValue(intCommand)), " ", _
                        RegexOptions.IgnoreCase)
                Next
            End If

            ' return the clean SQL
            Return strCleanSQL

        End Function

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Clears the results that have been cached by the module.
        ''' </summary>
        ''' <param name="ModuleID">The ID of the module to clear cached results for</param>
        ''' <history>
        '''     [anurse]        6/20/2006   Created
        '''     [anurse]        6/21/2006   Documented
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Shared Sub ClearCachedResults(ByVal ModuleID As Integer)
            DataCache.RemoveCache(String.Concat(CACHEKEY_Reports, ModuleID))
        End Sub

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Executes the query and returns the results for test purposes
        ''' </summary>
        ''' <remarks>
        ''' Designed to allow the module to quickly run scripts without the 
        ''' overhead of builting a report to wrap it
        ''' </remarks>
        ''' <param name="Query">The query to test</param>
        ''' <param name="Parameters">The parameters to pass to the query</param>
        ''' <history>
        '''     [anurse]        1/15/2007   Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Shared Function TestQuery(ByVal Query As String, ByVal Parameters As DbParameter()) As DataTable
            Dim dt As New DataTable("QueryResults")
            If Not String.IsNullOrEmpty(Query) Then
                Dim dr As IDataReader = ReportsDataProvider.Instance().ExecuteSQL(Query, Parameters)
                dt.Load(dr)
            End If
            Return dt
        End Function

        Public Shared Function BuildParameters(ByVal SrcModule As PortalModuleBase) As DbParameter()
            Return New SqlParameter() {BuildIntParam("@PortalID", SrcModule.PortalId), BuildIntParam("@TabID", SrcModule.TabId), _
                BuildIntParam("@ModuleID", SrcModule.ModuleId), BuildIntParam("@UserID", SrcModule.UserId)}
        End Function

        

#End Region

#Region " Private Methods "

        Private Shared Function GetSubDirectory(ByVal BaseDirectory As DirectoryInfo, ByVal Name As String) As DirectoryInfo
            Dim dirs As DirectoryInfo() = BaseDirectory.GetDirectories(Name)
            If dirs.Length < 1 Then
                Return Nothing
            End If
            Return dirs(0)
        End Function

        ' Internal version of SaveReportDefinition to allow SaveReport to use
        ' the same ModuleController instance for both method calls
        Private Shared Sub UpdateReportDefinition(ByVal ctrl As ModuleController, ByVal ModuleId As Integer, ByVal objReport As ReportInfo)
            ' Update the module settings with the data from the report
            ctrl.UpdateModuleSetting(ModuleId, ReportsController.SETTING_ReportTitle, objReport.Title)
            ctrl.UpdateModuleSetting(ModuleId, ReportsController.SETTING_ReportDescription, objReport.Description)
            ctrl.UpdateModuleSetting(ModuleId, ReportsController.SETTING_Query, objReport.Query)
            ctrl.UpdateModuleSetting(ModuleId, ReportsController.SETTING_ReportCreatedOn, objReport.CreatedOn)
            ctrl.UpdateModuleSetting(ModuleId, ReportsController.SETTING_ReportCreatedBy, objReport.CreatedBy)
        End Sub

        ' Internal version of SaveReportViewSettings to allow SaveReport to use
        ' the same ModuleController instance for both method calls
        Private Shared Sub UpdateReportView(ByVal ctrl As ModuleController, ByVal TabModuleId As Integer, ByVal objReport As ReportInfo)
            ' Update the cache duration if it is specified
            If Not objReport.CacheDuration.Equals(Null.NullInteger) Then
                ctrl.UpdateTabModuleSetting(TabModuleId, ReportsController.SETTING_CacheDuration, objReport.CacheDuration.ToString())
            End If

            ' Update the visualizer setting
            ctrl.UpdateTabModuleSetting(TabModuleId, ReportsController.SETTING_Visualizer, objReport.Visualizer)

            ' Update the tab module settings for the report if we have the tab module data
            ' which is indicated by a non-empty Visualizer name
            If Not String.IsNullOrEmpty(objReport.Visualizer) Then

                ' Build the visualizer setting prefix (see GetReport)
                Dim strVisPrefix = String.Format("{0}{1}_", PREFIX_Visualizer, objReport.Visualizer)

                ' For each visualizer setting
                For Each pair As KeyValuePair(Of String, String) In objReport.VisualizerSettings

                    ' Prepend the visualizer setting prefix and save to tab module settings
                    ctrl.UpdateTabModuleSetting(TabModuleId, _
                        String.Concat(strVisPrefix, pair.Key), pair.Value)
                Next
            End If
        End Sub

        Private Shared Function BuildIntParam(ByVal strName As String, ByVal iValue As Integer) As SqlParameter
            Dim sp As New SqlParameter(strName, SqlDbType.Int)
            sp.Value = iValue
            Return sp
        End Function

        Private Shared Sub WriteSettingsDictionary(ByVal xmlBuilder As StringBuilder, ByVal dict As Dictionary(Of String, String))
            For Each pair As KeyValuePair(Of String, String) In dict
                xmlBuilder.AppendFormat("<setting key=""{0}""><![CDATA[{1}]]></setting>{2}", pair.Key, pair.Value, Environment.NewLine)
            Next
        End Sub

        Private Shared Function ReadSettingsDictionary(ByVal xmlElem As XmlElement) As Dictionary(Of String, String)
            Dim dict As New Dictionary(Of String, String)()
            For Each settingNode As XmlNode In xmlElem.SelectNodes("setting")
                If settingNode.NodeType <> XmlNodeType.Element Then Continue For
                Dim settingElem As XmlElement = DirectCast(settingNode, XmlElement)
                dict.Add(settingElem.GetAttribute("key"), settingElem.InnerText)
            Next
            Return dict
        End Function

#End Region

#Region " Optional Interfaces "

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' GetSearchItems implements the ISearchable Interface
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <param name="ModInfo">The ModuleInfo for the module to be Indexed</param>
        ''' <history>
        ''' 	[anurse]	06/16/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Function GetSearchItems(ByVal ModInfo As Entities.Modules.ModuleInfo) As DotNetNuke.Services.Search.SearchItemInfoCollection Implements DotNetNuke.Entities.Modules.ISearchable.GetSearchItems

            ' Get the report
            Dim objReport As ReportInfo = GetReport(ModInfo.ModuleID)

            ' Execute the report and serialize it to Xml
            Dim objTable As DataTable = ExecuteReport(objReport, 0, True)
            Dim sbContent As New StringBuilder
            If objTable IsNot Nothing Then
                objTable.WriteXml(New System.IO.StringWriter(sbContent))
            End If

            ' Build a search item
            Dim objPortalSec As New PortalSecurity
            Dim objSearchItem As New SearchItemInfo(objReport.Title, _
                objPortalSec.InputFilter(objReport.Description, PortalSecurity.FilterFlag.NoMarkup), objReport.CreatedBy, _
                objReport.CreatedOn, ModInfo.ModuleID, String.Empty, sbContent.ToString())

            ' Add it to the collection and return
            Dim objSearchItems As New SearchItemInfoCollection()
            objSearchItems.Add(objSearchItem)
            Return objSearchItems

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' ExportModule implements the IPortable ExportModule Interface
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <param name="ModuleID">The Id of the module to be exported</param>
        ''' <history>
        ''' 	[anurse]	06/16/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Function ExportModule(ByVal ModuleID As Integer) As String Implements DotNetNuke.Entities.Modules.IPortable.ExportModule
            Dim objReport As ReportInfo = GetReport(ModuleID)
            If objReport Is Nothing Then
                Return String.Empty
            End If

            Dim xmlBuilder As New StringBuilder
            xmlBuilder.AppendFormat("<title><![CDATA[{0}]]></title>{1}", objReport.Title, Environment.NewLine)
            xmlBuilder.AppendFormat("<description><![CDATA[{0}]]></description>{1}", objReport.Description, Environment.NewLine)
            xmlBuilder.AppendFormat("<query><![CDATA[{0}]]></query>{1}", objReport.Query, Environment.NewLine)
            
            ' Can't do this because Import/Export module does not have a TabModuleID
            'xmlBuilder.AppendFormat("<cacheDuration>{0}</cacheDuration>{1}", objReport.CacheDuration)
            'xmlBuilder.AppendFormat("<visualizer name=""{0}"">", objReport.Visualizer)
            'WriteSettingsDictionary(xmlBuilder, objReport.VisualizerSettings)
            'xmlBuilder.AppendFormat("</visualizer>")

            Return xmlBuilder.ToString()
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' ImportModule implements the IPortable ImportModule Interface
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <param name="ModuleID">The Id of the module to be imported</param>
        ''' <param name="Content">The content to be imported</param>
        ''' <param name="Version">The version of the module to be imported</param>
        ''' <param name="UserId">The Id of the user performing the import</param>
        ''' <history>
        ''' 	[anurse]	06/16/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Sub ImportModule(ByVal ModuleID As Integer, ByVal Content As String, ByVal Version As String, ByVal UserId As Integer) Implements DotNetNuke.Entities.Modules.IPortable.ImportModule

            ' Check Access and Version
            Dim objUser As UserInfo = UserController.GetUser(-1, UserId, False)
            If objUser Is Nothing OrElse Not objUser.IsSuperUser Then Return

            Dim objNewReport As New ReportInfo
            Dim xmlDoc As New XmlDocument
            xmlDoc.LoadXml(String.Format("<content>{0}</content>", Content))
            Dim xmlRoot As XmlElement = xmlDoc.DocumentElement

            objNewReport.Title = XmlUtils.GetNodeValue(xmlRoot, "title", String.Empty)
            objNewReport.Description = XmlUtils.GetNodeValue(xmlRoot, "description", String.Empty)
            objNewReport.CreatedOn = DateTime.Now
            objNewReport.CreatedBy = UserId

            Dim ver As New Version(Version)
            If ver.Major = 4 AndAlso ver.Minor < 4 Then
                ' Legacy importing
                Dim queryString As String = XmlUtils.GetNodeValue(xmlRoot, "query", String.Empty)
                Dim queryBytes As Byte() = Convert.FromBase64String(queryString)
                objNewReport.Query = Encoding.Default.GetString(queryBytes)
            Else
                objNewReport.Query = XmlUtils.GetNodeValue(xmlRoot, "query", String.Empty)
            End If

            
            ' Can't do this because Import/Export module does not have a TabModuleID
            'Dim visElement As XmlElement = DirectCast(xmlRoot.SelectSingleNode("visualizer"), XmlElement)
            'objNewReport.CacheDuration = XmlUtils.GetNodeValue(xmlRoot, "cacheDuration", String.Empty)
            'objNewReport.Visualizer = visElement.GetAttribute("name")
            'objNewReport.VisualizerSettings = ReadSettingsDictionary(visElement)

            UpdateReportDefinition(ModuleID, objNewReport)
            ClearCachedResults(ModuleID)
        End Sub

#End Region

    End Class
End Namespace
