'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2006
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System.Data
Imports System.IO
Imports System.Web.UI

Imports DotNetNuke
Imports DotNetNuke.Entities.Modules
Imports DotNetNuke.UI.Utilities

Imports DotNetNuke.Modules.Reports.Installer

Namespace DotNetNuke.Modules.Reports

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' The Settings class manages Module Settings
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Partial Class Settings
        Inherits Entities.Modules.ModuleSettingsBase

        Private Property Report() As ReportInfo
            Get
                Return TryCast(ViewState("Report"), ReportInfo)
            End Get
            Set(ByVal value As ReportInfo)
                ViewState("Report") = value
            End Set
        End Property

#Region " Base Method Implementations "

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' LoadSettings loads the settings from the Database and displays them
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Overrides Sub LoadSettings()
            Try
                If Not Page.IsPostBack Then
                    Report = ReportsController.GetReport(ModuleId, TabModuleId)
                    If Report Is Nothing Then
                        Report = New ReportInfo()
                    End If

                    ' If the user has permission to see the Data Source Settings
                    If CheckPermissions() Then

                        ' Load the fields
                        txtTitle.Text = Report.Title
                        txtDescription.Text = Report.Description
                        txtQuery.Text = Report.Query
                        txtCacheDuration.Text = Report.CacheDuration.ToString()

                        ' Set the caching checkbox
                        If Report.CacheDuration <= 0 Then
                            chkCaching.Checked = False
                            Report.CacheDuration = 0
                        Else
                            chkCaching.Checked = True
                        End If

                        ' Update the cache duration text box visibility
                        UpdateCachingSpan()
                    End If

                    '' Set the visualizer to the default value if it is empty
                    'If String.IsNullOrEmpty(Report.Visualizer) Then
                    '    Report.Visualizer = "Grid"
                    'End If

                    ' Find the current visualizer in the drop down and select it
                    If Me.VisualizerDropDown.Items.FindByValue(Report.Visualizer) IsNot Nothing Then
                        Me.VisualizerDropDown.SelectedValue = Report.Visualizer
                        UpdateSelectedVisualizer()
                    End If
                End If
            Catch exc As Exception           'Module failed to load
                ProcessModuleLoadException(Me, exc)
            End Try
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' UpdateSettings saves the modified settings to the Database
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Overrides Sub UpdateSettings()
            Try
                ' Do not update report definition if the user is not a SuperUser
                If Me.UserInfo.IsSuperUser Then

                    ' Load the data source settings into the report
                    Dim objSecurity As New PortalSecurity
                    Report.Title = txtTitle.Text
                    Report.Description = objSecurity.InputFilter(txtDescription.Text, PortalSecurity.FilterFlag.NoScripting)

                    ' NOTE: Removed Query Filtering because it restricts fields like CreatedDate and only hosts can create queries anyway
                    Report.Query = txtQuery.Text

                    Report.CreatedBy = Me.UserId
                    Report.CreatedOn = DateTime.Now

                    ' Save the report definition
                    ReportsController.UpdateReportDefinition(ModuleId, Report)

                End If

                ' Non-SuperUsers can change TabModuleSettings (display settings)

                ' Update cache duration (0 => no caching)
                Dim duration As String = "0"
                If chkCaching.Checked Then
                    duration = txtCacheDuration.Text
                End If
                Report.CacheDuration = duration

                ' and Visualizer Settings
                Report.Visualizer = Me.VisualizerDropDown.SelectedValue()
                Report.VisualizerSettings.Clear()
                Dim settings as VisualizerSettingsBase = GetActiveVisualizer(VisualizerSettings.GetActiveView())
                If settings IsNot Nothing Then
                    settings.SaveSettings(Report.VisualizerSettings)
                End If

                ' Save the report view and clear the cache
                ReportsController.UpdateReportView(TabModuleId, Report)

                ' refresh cache
                SynchronizeModule()
            Catch exc As Exception           'Module failed to load
                ProcessModuleLoadException(Me, exc)
            End Try
        End Sub

#End Region

#Region " Event Handlers "

        Protected Sub Page_Init(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Init

            ' Setup the visualizers list
            SetupVisualizersList()

            DotNetNuke.UI.Utilities.ClientAPI.AddButtonConfirm(btnTestQuery, _
                Localization.GetString("btnTestQuery.Confirm", Me.LocalResourceFile))
            DotNetNuke.UI.Utilities.ClientAPI.AddButtonConfirm(btnShowXml, _
                Localization.GetString("btnTestQuery.Confirm", Me.LocalResourceFile))
        End Sub

        Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
            If IsPostBack Then UpdateSelectedVisualizer()
            If Not ClientAPI.BrowserSupportsFunctionality(ClientAPI.ClientFunctionality.DHTML) Then
                chkCaching.AutoPostBack = True
                AddHandler chkCaching.CheckedChanged, AddressOf chkCaching_CheckedChanged
            Else
                ReportsClientAPI.Import(Me.Page)
                ReportsClientAPI.ShowHideByCheckBox(chkCaching, spanCacheDuration)
            End If
            UpdateCachingSpan()
        End Sub

        Private Sub chkCaching_CheckedChanged(ByVal sender As Object, ByVal args As EventArgs)
            UpdateCachingSpan()
        End Sub

        Protected Sub btnShowXml_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnShowXml.Click
            rowXmlSource.Visible = False
            rowResults.Visible = False

            Try
                ' Run the query
                Dim tblResults As DataTable = ReportsController.TestQuery(txtQuery.Text, _
                    ReportsController.BuildParameters(Me))

                ' Serialize the results to Xml
                Dim swResults As New StringWriter
                tblResults.WriteXml(swResults)
                txtXmlSource.Text = swResults.ToString()
                rowXmlSource.Visible = True
            Catch ex As SqlException
                ' Format an error message
                lblQueryResults.Text = String.Format(Localization.GetString("TestQueryFail.Message", Me.LocalResourceFile), ex.LineNumber, ex.Message)
                lblQueryResults.CssClass = "NormalRed"
                imgQueryResults.ImageUrl = "~/images/red-error.gif"
                rowResults.Visible = True
                rowXmlSource.Visible = False
            End Try
        End Sub

        Protected Sub btnHideXmlSource_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnHideXmlSource.Click
            rowXmlSource.Visible = False
        End Sub

        Protected Sub btnTestQuery_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnTestQuery.Click
            rowXmlSource.Visible = False
            rowResults.Visible = False

            Try
                ' Run the query
                Dim iResults As Integer = ReportsController.TestQuery(txtQuery.Text, _
                    ReportsController.BuildParameters(Me)).Rows.Count

                ' Format a success message
                lblQueryResults.Text = String.Format(Localization.GetString("TestQuerySuccess.Message", Me.LocalResourceFile), iResults)
                lblQueryResults.CssClass = "NormalBold"
                imgQueryResults.ImageUrl = "~/images/green-ok.gif"
            Catch ex As SqlException
                ' Format an error message
                lblQueryResults.Text = String.Format(Localization.GetString("TestQueryFail.Message", Me.LocalResourceFile), ex.LineNumber, ex.Message)
                lblQueryResults.CssClass = "NormalRed"
                imgQueryResults.ImageUrl = "~/images/red-error.gif"
            End Try

            ' Display the results/error message
            rowResults.Visible = True
        End Sub

        Protected Sub btnHideTestResults_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnHideTestResults.Click
            rowResults.Visible = False
        End Sub

        Protected Sub btnUpload_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnUpload.Click
            Dim reader As New StreamReader(fileQuery.PostedFile.InputStream)
            txtQuery.Text = reader.ReadToEnd()
        End Sub

#End Region

#Region " Private Methods "

        Private Function CheckPermissions() As Boolean
            If Not Me.UserInfo.IsSuperUser Then
                ReportsSettingsMultiView.SetActiveView(AccessDeniedView)
                Return False
            Else
                ReportsSettingsMultiView.SetActiveView(SuperUserView)
                Return True
            End If
        End Function

        Private Sub SetupVisualizersList()
            ' Find the Renderers Folder
            Dim sVisualizersFolder = Server.MapPath(Me.ResolveUrl("Visualizers"))

            Me.VisualizerDropDown.Items.Clear()

            Me.VisualizerDropDown.Items.Add(New ListItem( _
                Localization.GetString("NoVisualizer.Text", Me.LocalResourceFile), _
                String.Empty))

            ' Get the list of visualizers
            For Each visManifest As PackageManifest In PackageInstaller.GetExtensions(Server.MapPath(Me.TemplateSourceDirectory), False)
                ' If the visualizer is not designed for this version, skip it
                If visManifest.ModuleVersion.Major <> 4 OrElse visManifest.ModuleVersion.Minor <> 5 Then
                    Continue For
                End If

                ' Get the folder
                Dim sVisualizerLRFolder As String = Me.ResolveUrl(String.Format("Visualizers/{0}/App_LocalResources", visManifest.Name))
                Dim sVisualizerRESX As String = String.Format("{0}/{1}.resx", sVisualizerLRFolder, VisualizerControlBase.FILENAME_VisualizerASCX)
                If File.Exists(Server.MapPath(sVisualizerRESX)) Then
                    Dim sVisualizerName As String = Localization.GetString("VisualizerName.Text", sVisualizerRESX)
                    If Not String.IsNullOrEmpty(sVisualizerName) Then
                        Me.VisualizerDropDown.Items.Add(New ListItem(String.Format(Localization.GetString("Visualizer.Text", Me.LocalResourceFile), sVisualizerName), visManifest.Name))

                        Dim ctlVisualizerSettings As VisualizerSettingsBase = LoadVisualizer(visManifest.Name, sVisualizerLRFolder)
                        If ctlVisualizerSettings Is Nothing Then Continue For
                        Dim vwVisualizerSettings As New View()
                        vwVisualizerSettings.ID = String.Format("vw{0}", visManifest.Name)
                        vwVisualizerSettings.Controls.Add(ctlVisualizerSettings)

                        VisualizerSettings.Views.Add(vwVisualizerSettings)
                    End If
                End If
            Next
        End Sub

        Private Sub UpdateSelectedVisualizer()
            Dim sViewName As String
            If Not String.IsNullOrEmpty(Me.VisualizerDropDown.SelectedValue) Then
                sViewName = String.Format("vw{0}", Me.VisualizerDropDown.SelectedValue)
            End If

            Dim activeView As View = VisualizerSettings.GetActiveView()
            If activeView IsNot Nothing AndAlso activeView.ID.Equals(sViewName, StringComparison.OrdinalIgnoreCase) Then
                Return
            End If

            Dim rv As View = Nothing
            If String.IsNullOrEmpty(sViewName) Then
                rv = Me.VisualizerNotConfiguredView
            Else
                rv = DirectCast(VisualizerSettings.FindControl(sViewName), View)
            End If

            VisualizerSettings.SetActiveView(rv)

            Dim settings As VisualizerSettingsBase = GetActiveVisualizer(rv)
            If settings IsNot Nothing Then
                settings.LoadSettings(Report.VisualizerSettings)
            End If
        End Sub

        Private Function GetActiveVisualizer(ByVal ActiveView As View) As VisualizerSettingsBase
            If ActiveView.Equals(Me.VisualizerNotConfiguredView) Then Return Nothing
            Dim sCtlName As String = String.Format("ctl{0}", Me.VisualizerDropDown.SelectedValue)
            Return DirectCast(ActiveView.FindControl(sCtlName), VisualizerSettingsBase)
        End Function

        Private Function LoadVisualizer(ByVal DirName As String, ByVal visualizerLRFolder As String) As VisualizerSettingsBase
            ' Load the control and inject it into the MultiView
            Try
                Dim sVisualizerSettingsControl As String = Me.ResolveUrl(String.Format("Visualizers/{0}/{1}", DirName, VisualizerSettingsBase.FILENAME_SettingsASCX))
                Dim ctlVisualizerSettings As VisualizerSettingsBase = DirectCast(LoadControl(sVisualizerSettingsControl), VisualizerSettingsBase)
                ctlVisualizerSettings.ID = String.Format("ctl{0}", DirName)
                ctlVisualizerSettings.ParentModule = Me
                Return ctlVisualizerSettings
            Catch ex As Exception
                Return Nothing
            End Try
        End Function

        Protected Sub VisualizerDropDown_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles VisualizerDropDown.SelectedIndexChanged
            UpdateSelectedVisualizer()
        End Sub

        Private Sub UpdateCachingSpan()
            If chkCaching.Checked Then
                spanCacheDuration.Style(HtmlTextWriterStyle.Display) = String.Empty
            Else
                spanCacheDuration.Style(HtmlTextWriterStyle.Display) = "none"
            End If
        End Sub

#End Region
    End Class

End Namespace

