'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2006
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System.IO
Imports System.Web.UI

Imports DotNetNuke
Imports DotNetNuke.Entities.Modules
Imports DotNetNuke.UI.Utilities

Namespace DotNetNuke.Modules.Reports

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' The Settings class manages Module Settings
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Partial Class Settings
        Inherits Entities.Modules.ModuleSettingsBase

#Region " Base Method Implementations "

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' LoadSettings loads the settings from the Database and displays them
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Overrides Sub LoadSettings()
            Try
                If Not Page.IsPostBack Then
                    If CheckPermissions() Then
                        Dim objReport As ReportInfo = ReportsController.GetReport(ModuleId)
                        If objReport IsNot Nothing Then
                            txtTitle.Text = objReport.Title
                            txtDescription.Text = objReport.Description
                            txtQuery.Text = objReport.Query
                        End If
                        If TabModuleSettings(ReportsController.SETTING_CacheDuration) Is Nothing Then
                            txtCacheDuration.Text = "0"
                        Else
                            txtCacheDuration.Text = Convert.ToString(TabModuleSettings(ReportsController.SETTING_CacheDuration))
                        End If

                        If String.IsNullOrEmpty(txtCacheDuration.Text) OrElse txtCacheDuration.Text = "0" Then
                            chkCaching.Checked = False
                        Else
                            chkCaching.Checked = True
                        End If
                        UpdateCachingSpan()
                    End If

                    Dim sRenderer As String = Convert.ToString(TabModuleSettings(ReportsController.SETTING_Renderer))
                    If String.IsNullOrEmpty(sRenderer) Then
                        sRenderer = "Grid"
                    End If
                    If Me.RendererDropDown.Items.FindByValue(sRenderer) IsNot Nothing Then
                        Me.RendererDropDown.SelectedValue = sRenderer
                        UpdateSelectedRenderer()
                    End If
                End If
            Catch exc As Exception           'Module failed to load
                ProcessModuleLoadException(Me, exc)
            End Try
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' UpdateSettings saves the modified settings to the Database
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Overrides Sub UpdateSettings()
            Try
                ' Do not update report specification if the user is not a SuperUser
                If Me.UserInfo.IsSuperUser Then
                    Dim objReport As New ReportInfo()
                    Dim objSecurity As New PortalSecurity
                    objReport.Title = txtTitle.Text
                    objReport.Description = objSecurity.InputFilter(txtDescription.Text, PortalSecurity.FilterFlag.NoScripting)
                    'Removed Query Filtering because it restricts fields like CreatedDate and only hosts can create queries anyway
                    objReport.Query = txtQuery.Text
                    objReport.CreatedBy = Me.UserId
                    objReport.CreatedOn = DateTime.Now

                    ReportsController.SetReport(ModuleId, objReport)
                    ReportsController.ClearCachedResults(Me.ModuleId)
                End If

                ' Non-SuperUsers can change TabModuleSettings (display settings)
                Dim objModuleController As New DotNetNuke.Entities.Modules.ModuleController
                Dim duration As String = "0"
                If chkCaching.Checked Then
                    duration = txtCacheDuration.Text
                End If
                objModuleController.UpdateTabModuleSetting(TabModuleId, ReportsController.SETTING_CacheDuration, duration)
                objModuleController.UpdateTabModuleSetting(TabModuleId, ReportsController.SETTING_Renderer, Me.RendererDropDown.SelectedValue)

                GetActiveRenderer(RendererSettings.GetActiveView()).UpdateSettings()

                ' refresh cache
                SynchronizeModule()
            Catch exc As Exception           'Module failed to load
                ProcessModuleLoadException(Me, exc)
            End Try
        End Sub

#End Region

#Region " Event Handlers "

        Protected Sub Page_Init(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Init
            SetupRenderersList()
        End Sub

        Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
            UpdateSelectedRenderer()
            If Not ClientAPI.BrowserSupportsFunctionality(ClientAPI.ClientFunctionality.DHTML) Then
                chkCaching.AutoPostBack = True
                AddHandler chkCaching.CheckedChanged, AddressOf chkCaching_CheckedChanged
            Else
                ReportsClientAPI.Import(Me.Page)
                ReportsClientAPI.ShowHideByCheckBox(chkCaching, spanCacheDuration)
            End If
        End Sub

        Private Sub chkCaching_CheckedChanged(ByVal sender As Object, ByVal args As EventArgs)
            UpdateCachingSpan()
        End Sub

#End Region

#Region " Private Methods "

        Private Function CheckPermissions() As Boolean
            If Not Me.UserInfo.IsSuperUser Then
                ReportsSettingsMultiView.SetActiveView(AccessDeniedView)
                Return False
            Else
                ReportsSettingsMultiView.SetActiveView(SuperUserView)
                Return True
            End If
        End Function

        Private Sub SetupRenderersList()
            ' Find the Renderers Folder
            Dim sRenderersFolder = Server.MapPath(Me.ResolveUrl("Visualizers"))

            ' Get the subfolders
            For Each sFolder As String In Directory.GetDirectories(sRenderersFolder)
                ' Find the [dir]/App_LocalResources/Visualizer.ascx.resx
                Dim dir As New DirectoryInfo(sFolder)
                Dim sRendererLRFolder As String = Me.ResolveUrl(String.Format("Visualizers/{0}/App_LocalResources", dir.Name))
                Dim sRendererRESX As String = String.Format("{0}/Visualizer.ascx.resx", sRendererLRFolder)
                If File.Exists(Server.MapPath(sRendererRESX)) Then
                    Dim sRendererName As String = Localization.GetString("VisualizerName.Text", sRendererRESX)
                    If Not String.IsNullOrEmpty(sRendererName) Then
                        Me.RendererDropDown.Items.Add(New ListItem(String.Format(Localization.GetString("Visualizer.Text", Me.LocalResourceFile), sRendererName), dir.Name))

                        Dim ctlRendererSettings As ModuleSettingsBase = LoadRenderer(dir.Name, sRendererLRFolder)
                        Dim vwRendererSettings As New View()
                        vwRendererSettings.ID = String.Format("vw{0}", dir.Name)
                        vwRendererSettings.Controls.Add(ctlRendererSettings)

                        RendererSettings.Views.Add(vwRendererSettings)
                    End If
                End If
            Next
        End Sub

        Private Sub UpdateSelectedRenderer()
            Dim sViewName As String = String.Format("vw{0}", Me.RendererDropDown.SelectedValue)
            Dim activeView As View = RendererSettings.GetActiveView()
            If activeView IsNot Nothing AndAlso activeView.ID.Equals(sViewName, StringComparison.OrdinalIgnoreCase) Then
                Return
            End If

            Dim rv As View = DirectCast(RendererSettings.FindControl(sViewName), View)
            RendererSettings.SetActiveView(rv)
            GetActiveRenderer(rv).LoadSettings()
        End Sub

        Private Function GetActiveRenderer(ByVal ActiveView As View) As ModuleSettingsBase
            Dim sCtlName As String = String.Format("ctl{0}", Me.RendererDropDown.SelectedValue)
            Return DirectCast(ActiveView.FindControl(sCtlName), ModuleSettingsBase)
        End Function

        Private Function LoadRenderer(ByVal DirName As String, ByVal rendererLRFolder As String) As ModuleSettingsBase
            ' Load the control and inject it into the MultiView
            Dim sRendererSettingsControl As String = Me.ResolveUrl(String.Format("Visualizers/{0}/Settings.ascx", DirName))
            Dim ctlRendererSettings As ModuleSettingsBase = DirectCast(LoadControl(sRendererSettingsControl), ModuleSettingsBase)
            ctlRendererSettings.ID = String.Format("ctl{0}", DirName)
            ctlRendererSettings.ModuleId = ModuleId
            ctlRendererSettings.TabModuleId = TabModuleId
            ctlRendererSettings.LocalResourceFile = String.Format("{0}/Settings.ascx.resx", rendererLRFolder)
            Return ctlRendererSettings
        End Function

        Protected Sub RendererDropDown_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles RendererDropDown.SelectedIndexChanged
            UpdateSelectedRenderer()
        End Sub

        Private Sub UpdateCachingSpan()
            If chkCaching.Checked Then
                spanCacheDuration.Style(HtmlTextWriterStyle.Display) = String.Empty
            Else
                spanCacheDuration.Style(HtmlTextWriterStyle.Display) = "none"
            End If
        End Sub

#End Region
        
    End Class

End Namespace

