'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2006
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'
Imports DotNetNuke
Imports DotNetNuke.Entities.Modules

Imports System.Web.UI
Imports System.Collections.Generic
Imports System.Reflection
Imports UISkin = DotNetNuke.UI.Skins.Skin
Imports System

Namespace DotNetNuke.Modules.Reports

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' The SettingsUtil class provides utilities for accessing settings
    ''' </summary>
    ''' <history>
    '''     [anurse]     08/31/2006    Documented
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Class SettingsUtil

        ' Gets a setting from the dictionary provided, or returns defValue if the setting is null
        Public Shared Function GetDictionarySetting(Of T)(ByVal settings As Dictionary(Of String, String), ByVal key As String, ByVal defValue As T) As T
            If Not settings.ContainsKey(key) Then Return defValue
            Dim sVal As String = settings(key)
            Dim retValue As T = defValue
            If Not String.IsNullOrEmpty(sVal) Then
                Try
                    retValue = Convert.ChangeType(sVal, GetType(T))
                Catch ex As Exception
                    retValue = defValue
                End Try
            End If
            Return retValue
        End Function

        ' Gets a setting from the hashtable provided, or returns defValue if the setting is null
        Public Shared Function GetHashtableSetting(Of T)(ByVal settings As Hashtable, ByVal key As String, ByVal defValue As T) As T
            If Not settings.ContainsKey(key) Then Return defValue
            Dim o As Object = settings(key)
            Dim retValue As T = defValue
            If o IsNot Nothing Then
                Try
                    retValue = Convert.ChangeType(o, GetType(T))
                Catch ex As Exception
                    retValue = defValue
                End Try
            End If
            Return retValue
        End Function

    End Class

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' The SettingsUtil class provides general utilities
    ''' </summary>
    ''' <history>
    '''     [anurse]     08/31/2006    Documented
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Class Utilities

        Public Shared Function MapFileIdPath(ByVal PortalSettings As PortalSettings, ByVal FileId As String) As String
            If Not String.IsNullOrEmpty(FileId) AndAlso FileId.ToLower().StartsWith("fileid=", StringComparison.OrdinalIgnoreCase) Then
                Dim intFileId As Integer = Int32.Parse(FileId.Substring(7))
                Return MapFileIdPath(PortalSettings, intFileId)
            End If
            Return String.Empty
        End Function

        Public Shared Function MapFileIdPath(ByVal PortalSettings As PortalSettings, ByVal FileId As Integer) As String
            Dim objFileCtrl As New DotNetNuke.Services.FileSystem.FileController()
            Dim objFile As DotNetNuke.Services.FileSystem.FileInfo = objFileCtrl.GetFileById(FileId, PortalSettings.PortalId)
            Dim sProtectedExtension As String = String.Empty
            If objFile.StorageLocation = DotNetNuke.Services.FileSystem.FolderController.StorageLocationTypes.DatabaseSecure Then
                Return String.Empty
            ElseIf objFile.StorageLocation = DotNetNuke.Services.FileSystem.FolderController.StorageLocationTypes.SecureFileSystem Then
                sProtectedExtension = DotNetNuke.Common.Globals.glbProtectedExtension
            End If
            Return String.Concat(PortalSettings.HomeDirectoryMapPath, objFile.Folder, objFile.FileName, sProtectedExtension)
        End Function

        Public Shared Function ParseColorString(ByVal colorString As String) As Color
            Dim bIsShortString As Boolean = False
            If String.IsNullOrEmpty(colorString) Then Return Color.Black
            If colorString.StartsWith("#") Then colorString = colorString.Substring(1)
            If colorString.Length = 3 Then
                bIsShortString = True
            ElseIf colorString.Length <> 6 Then
                Return Color.Black
            End If

            Dim clr As Color = Color.Black
            Try
                Dim iR As Integer = ParseHexCode(colorString, bIsShortString, 0)
                Dim iG As Integer = ParseHexCode(colorString, bIsShortString, 1)
                Dim iB As Integer = ParseHexCode(colorString, bIsShortString, 2)
                clr = Color.FromArgb(iR, iG, iB)
            Catch fe As FormatException
                Return Color.Black
            End Try
            Return clr
        End Function

        Private Shared Function ParseHexCode(ByVal colorString As String, ByVal bIsShortString As Boolean, ByVal Position As Integer) As Integer
            Dim len As Integer = 2
            If bIsShortString Then
                len = 1
            Else
                Position *= 2
            End If
            Return Int32.Parse(colorString.Substring(Position, len), NumberStyles.HexNumber)
        End Function

    End Class

End Namespace