'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2006
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'
Imports DotNetNuke

Imports System.IO
Imports System.Web.UI
Imports System.Collections.Generic
Imports System.Reflection

Imports UISkin = DotNetNuke.UI.Skins.Skin

Namespace DotNetNuke.Modules.Reports

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' The ViewReports class displays the content
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Partial Class ViewReports
        Inherits Entities.Modules.PortalModuleBase
        Implements Entities.Modules.IActionable

        Private Function VisualizerFolderExists(ByVal visualizerFolder As String) As Boolean

            Return Directory.Exists(Server.MapPath(Me.ResolveUrl(String.Format("Visualizers/{0}", visualizerFolder))))

        End Function
        Private Sub LoadVisualizer()

            ' Check for a visualizer
            Dim sVisualizerFolder As String = Convert.ToString(Settings.Item(ReportsController.SETTING_Visualizer))
            If String.IsNullOrEmpty(sVisualizerFolder) OrElse Not VisualizerFolderExists(sVisualizerFolder) Then
                If VisualizerFolderExists("Grid") Then
                    ' Default to grid if its installed, otherwise default to none selected
                    ' This should cover most upgrades from pre-Visualizer versions as long as the
                    ' user sticks with the default and installs Grid, if not, then they are advanced
                    ' enough to know what to do.

                    sVisualizerFolder = "Grid"
                ElseIf IsEditable Then
                    DotNetNuke.UI.Skins.Skin.AddModuleMessage(Me, _
                        Localization.GetString("VisualizerNotConfigured.Text", _
                        Me.LocalResourceFile), _
                        Skins.Controls.ModuleMessage.ModuleMessageType.RedError)
                End If
            End If

            ' If the visualizer folder is now non-empty
            If Not String.IsNullOrEmpty(sVisualizerFolder) Then
                ' Find the visualizer control
                Dim sVisualizerControl As String = Me.ResolveUrl(String.Format("Visualizers/{0}/{1}", sVisualizerFolder, VisualizerControlBase.FILENAME_VisualizerASCX))
                If Not File.Exists(Server.MapPath(sVisualizerControl)) Then
                    If IsEditable Then
                        DotNetNuke.UI.Skins.Skin.AddModuleMessage(Me, _
                            Localization.GetString("VisualizerDoesNotExist.Text", _
                            Me.LocalResourceFile), _
                            Skins.Controls.ModuleMessage.ModuleMessageType.RedError)
                    End If
                Else
                    Dim ctlVisualizer As VisualizerControlBase = Nothing
                    Try
                        ' Load the visualizer control
                        ctlVisualizer = TryCast(Me.LoadControl(sVisualizerControl), VisualizerControlBase)
                    Catch ex As Exception
                        ctlVisualizer = Nothing
                    End Try
                    If ctlVisualizer IsNot Nothing Then
                        ctlVisualizer.ID = "Visualizer"
                        ctlVisualizer.ParentModule = Me
                        If ctlVisualizer.AutoLoadReport Then
                            ctlVisualizer.LoadReport()
                            ctlVisualizer.ExecuteReport()
                        End If

                        Me.Controls.Clear()
                        Me.Controls.Add(ctlVisualizer)
                    ElseIf IsEditable Then
                        DotNetNuke.UI.Skins.Skin.AddModuleMessage(Me, _
                            Localization.GetString("VisualizerLoadError.Text", _
                            Me.LocalResourceFile), _
                            Skins.Controls.ModuleMessage.ModuleMessageType.RedError)
                    End If
                End If
            End If
        End Sub

        Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
            LoadVisualizer()
        End Sub

        Public ReadOnly Property ModuleActions() As Entities.Modules.Actions.ModuleActionCollection Implements Entities.Modules.IActionable.ModuleActions
            Get
                Dim actions As New Entities.Modules.Actions.ModuleActionCollection
                actions.Add(Me.GetNextActionID(), _
                    Localization.GetString("ManagePackages.Action", Me.LocalResourceFile), _
                    "", "", Me.ResolveUrl("images/ManagePackages.gif"), _
                    EditUrl("ManagePackages"), False, SecurityAccessLevel.Host, True, False)
                Return actions
            End Get
        End Property
    End Class

End Namespace
