'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2006
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'
Imports DotNetNuke
Imports DotNetNuke.Entities.Modules

Imports System.Web.UI
Imports System.Collections.Generic
Imports System.Reflection
Imports UISkin = DotNetNuke.UI.Skins.Skin
Imports System

Namespace DotNetNuke.Modules.Reports

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' The VisualizerControlBase class provides a base class for Visualizer Controls
    ''' </summary>
    ''' <remarks>
    ''' Visualizer Controls are the component of the visualizer that actually renders the report data.
    ''' </remarks>
    ''' <history>
    '''     [anurse]     08/31/2006    Documented
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Class VisualizerControlBase
        Inherits ReportsControlBase

        Public Const FILENAME_VisualizerASCX As String = "Visualizer.ascx"

        Private _reportResults As DataTable
        Public Property ReportResults() As DataTable
            Get
                Return _reportResults
            End Get
            Set(ByVal value As DataTable)
                _reportResults = value
            End Set
        End Property

        Private _report As ReportInfo
        Public Property Report() As ReportInfo
            Get
                Return _report
            End Get
            Set(ByVal value As ReportInfo)
                _report = value
            End Set
        End Property

        Public Overridable ReadOnly Property AutoLoadReport() As Boolean
            Get
                Return True
            End Get
        End Property

        Protected Overrides ReadOnly Property ASCXFileName() As Object
            Get
                Return FILENAME_VisualizerASCX
            End Get
        End Property

        Public Sub LoadReport()
            Report = ReportsController.GetReport(Me.ParentModule.ModuleId, Me.ParentModule.TabModuleId)
        End Sub

        Public Sub ExecuteReport()
            ExecuteReport(False, True)
        End Sub

        Public Sub ExecuteReport(ByVal ThrowOnError As Boolean)
            ExecuteReport(ThrowOnError, True)
        End Sub

        Public Sub ExecuteReport(ByVal ThrowOnError As Boolean, ByVal ShowErrorMessage As Boolean)
            Try
                ReportResults = ReportsController.ExecuteReport(Report, Me.ParentModule.CacheKey, False, ReportsController.BuildParameters(Me.ParentModule))
            Catch sqlExc As SqlException
                If ShowErrorMessage Then
                    If Me.ParentModule.UserInfo.IsSuperUser Then
                        UISkin.AddModuleMessage(Me.ParentModule, _
                        String.Format(Localization.GetString("HostSQLError.Message", Me.ParentModule.LocalResourceFile), sqlExc.LineNumber, sqlExc.Message), _
                            Skins.Controls.ModuleMessage.ModuleMessageType.RedError)
                    Else
                        UISkin.AddModuleMessage(Me.ParentModule, Localization.GetString("AdminSQLError.Message", Me.LocalResourceFile), _
                            Skins.Controls.ModuleMessage.ModuleMessageType.YellowWarning)
                    End If
                End If

                If ThrowOnError Then
                    Throw
                End If
            End Try
        End Sub

        Protected Function ValidateQuery() As Boolean
            Return ValidateQuery(True)
        End Function

        Protected Function ValidateQuery(ByVal ShowMessage As Boolean) As Boolean
            If String.IsNullOrEmpty(Me.Report.Query) Then
                If ShowMessage Then
                    If Me.ParentModule.UserInfo.IsSuperUser Then
                        UISkin.AddModuleMessage(Me.ParentModule, Localization.GetString("HostNoReport.Message", Me.ParentModule.LocalResourceFile), _
                            Skins.Controls.ModuleMessage.ModuleMessageType.YellowWarning)
                    ElseIf Me.ParentModule.IsEditable Then
                        UISkin.AddModuleMessage(Me.ParentModule, Localization.GetString("AdminNoReport.Message", Me.ParentModule.LocalResourceFile), _
                            Skins.Controls.ModuleMessage.ModuleMessageType.YellowWarning)
                    End If
                End If
                Return False
            Else
                Return True
            End If
        End Function

        Protected Function ValidateResults() As Boolean
            Return ValidateResults(True)
        End Function

        Protected Function ValidateResults(ByVal ShowMessage As Boolean) As Boolean
            If Me.ReportResults Is Nothing OrElse Me.ReportResults.Rows.Count = 0 Then
                If ShowMessage Then
                    If Me.ParentModule.IsEditable Then
                        UISkin.AddModuleMessage(Me.ParentModule, Localization.GetString("NoResults.Message", Me.ParentModule.LocalResourceFile), _
                            Skins.Controls.ModuleMessage.ModuleMessageType.YellowWarning)
                    End If
                End If
                Return False
            Else
                Return True
            End If
        End Function

        'Private Sub EnsureResults()
        '    If _reportResults IsNot Nothing Then Return
        '    Dim objModuleController As New DotNetNuke.Entities.Modules.ModuleController
        '    Dim objTabModuleSettings As Hashtable = objModuleController.GetTabModuleSettings(Me.ParentModule.TabModuleId)
        '    Dim cacheDuration As Integer = SettingsUtil.GetTabModuleSetting(Of Integer)(objTabModuleSettings, ReportsController.SETTING_CacheDuration, 0)
        '    Dim objReport As ReportInfo = ReportsController.GetReport(Me.ParentModule.ModuleId)
        '    Dim params As SqlParameter() = BuildParameters()
        '    _reportResults = ReportsController.ExecuteReport(objReport, cacheDuration, False, params)
        'End Sub

        'Private Sub EnsureReport()
        '    If _report IsNot Nothing Then Return
        '    _report = ReportsController.GetReport(Me.ParentModule.ModuleId)
        'End Sub

        

    End Class

End Namespace
