'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2006
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'
Imports DotNetNuke
Imports DotNetNuke.Entities.Modules

Imports System.Web.UI
Imports System.Collections.Generic
Imports System.Reflection
Imports UISkin = DotNetNuke.UI.Skins.Skin
Imports System

Namespace DotNetNuke.Modules.Reports

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' The VisualizerControlBase class provides a base class for Visualizer Controls
    ''' </summary>
    ''' <remarks>
    ''' Visualizer Controls are the component of the visualizer that actually renders the report data.
    ''' </remarks>
    ''' <history>
    '''     [anurse]     08/31/2006    Documented
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public Class VisualizerControlBase
        Inherits UserControlBase

        Public Const glbRendererASCX As String = "Visualizer.ascx"

        Private _parentModule As PortalModuleBase
        Public Property ParentModule() As PortalModuleBase
            Get
                Return _parentModule
            End Get
            Set(ByVal Value As PortalModuleBase)
                _parentModule = Value
            End Set
        End Property

        Private _localResourceFile As String
        Public ReadOnly Property LocalResourceFile() As String
            Get
                If String.IsNullOrEmpty(_localResourceFile) Then
                    _localResourceFile = Me.TemplateSourceDirectory & "/" & Services.Localization.Localization.LocalResourceDirectory & "/" & glbRendererASCX
                End If
                Return _localResourceFile
            End Get
        End Property

        Private _reportResults As DataTable
        Protected ReadOnly Property ReportResults() As DataTable
            Get
                EnsureResults()
                Return _reportResults
            End Get
        End Property

        Private _report As ReportInfo
        Protected ReadOnly Property Report() As ReportInfo
            Get
                EnsureReport()
                Return _report
            End Get
        End Property

        Private Sub EnsureResults()
            If _reportResults IsNot Nothing Then Return
            Dim objModuleController As New DotNetNuke.Entities.Modules.ModuleController
            Dim objTabModuleSettings As Hashtable = objModuleController.GetTabModuleSettings(Me.ParentModule.TabModuleId)
            Dim cacheDuration As Integer = SettingsUtil.GetTabModuleSetting(Of Integer)(objTabModuleSettings, ReportsController.SETTING_CacheDuration, 0)
            Dim objReport As ReportInfo = ReportsController.GetReport(Me.ParentModule.ModuleId)
            Dim params As SqlParameter() = BuildParameters()
            _reportResults = ReportsController.ExecuteReport(objReport, cacheDuration, False, params)
        End Sub

        Private Sub EnsureReport()
            If _report IsNot Nothing Then Return
            _report = ReportsController.GetReport(Me.ParentModule.ModuleId)
        End Sub

        Private Function BuildParameters() As SqlParameter()
            ' Simple set of four params for now
            Return New SqlParameter() {BuildIntParam("@PortalID", Me.ParentModule.PortalId), BuildIntParam("@TabID", Me.ParentModule.TabId), _
                BuildIntParam("@ModuleID", Me.ParentModule.ModuleId), BuildIntParam("@UserID", Me.ParentModule.UserId)}
        End Function

        Private Function BuildIntParam(ByVal strName As String, ByVal iValue As Integer) As SqlParameter
            Dim sp As New SqlParameter(strName, SqlDbType.Int)
            sp.Value = iValue
            Return sp
        End Function

    End Class

End Namespace
